package com.jiuqi.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

/* loaded from: com_jiuqi_util.jar:com/jiuqi/util/SortedList.class */
public final class SortedList implements List {
    private List items;
    private Comparator comparator;
    private int[] findStub;

    public SortedList() {
        this.findStub = new int[1];
        this.items = new ArrayList();
    }

    public SortedList(int initialCapacity) {
        this.findStub = new int[1];
        this.items = new ArrayList(initialCapacity);
    }

    public SortedList(Comparator comparator) {
        this.findStub = new int[1];
        this.items = new ArrayList();
        this.comparator = comparator;
    }

    public SortedList(int initalCapacity, Comparator comparator) {
        this.findStub = new int[1];
        this.items = new ArrayList(initalCapacity);
        this.comparator = comparator;
    }

    public int compareObjects(Object o1, Object o2) {
        if (this.comparator != null) {
            return this.comparator.compare(o1, o2);
        }
        if (o1 instanceof Comparable) {
            return ((Comparable) o1).compareTo(o2);
        }
        if (o2 instanceof Comparable) {
            return -((Comparable) o2).compareTo(o1);
        }
        throw new UnsupportedOperationException("对象比较时无法转化为Comparable接口！");
    }

    @Override // java.util.List, java.util.Collection
    public boolean add(Object o) {
        int[] index = new int[1];
        find(o, index);
        this.items.add(index[0], o);
        return true;
    }

    @Override // java.util.List
    public void add(int index, Object element) {
        add(element);
    }

    public void _add(int index, Object element) {
        this.items.add(index, element);
    }

    @Override // java.util.List, java.util.Collection
    public boolean addAll(Collection c) {
        List newItems = new ArrayList(c);
        reset(newItems);
        return true;
    }

    @Override // java.util.List
    public boolean addAll(int index, Collection c) {
        return addAll(c);
    }

    @Override // java.util.List, java.util.Collection
    public void clear() {
        this.items.clear();
    }

    @Override // java.util.List, java.util.Collection
    public boolean contains(Object o) {
        return find(o, new int[1]);
    }

    @Override // java.util.List, java.util.Collection
    public boolean containsAll(Collection c) {
        Iterator itr = c.iterator();
        while (itr.hasNext()) {
            if (!contains(itr.next())) {
                return false;
            }
        }
        return true;
    }

    @Override // java.util.List
    public Object get(int index) {
        return this.items.get(index);
    }

    @Override // java.util.List
    public int indexOf(Object o) {
        int low = 0;
        int high = size() - 1;
        while (low <= high) {
            int mid = (low + high) / 2;
            int flag = compareObjects(o, get(mid));
            if (flag == 0) {
                if (mid == low) {
                    return mid;
                }
                high = mid;
            } else if (flag > 0) {
                low = mid + 1;
            } else {
                high = mid - 1;
            }
        }
        return -1;
    }

    @Override // java.util.List, java.util.Collection
    public boolean isEmpty() {
        return this.items.isEmpty();
    }

    @Override // java.util.List, java.util.Collection, java.lang.Iterable
    public Iterator iterator() {
        return this.items.iterator();
    }

    @Override // java.util.List
    public int lastIndexOf(Object o) {
        int low = 0;
        int high = size() - 1;
        while (low <= high) {
            int mid = ((low + high) + 1) / 2;
            int flag = compareObjects(o, get(mid));
            if (flag == 0) {
                if (mid == high) {
                    return mid;
                }
                low = mid;
            } else if (flag > 0) {
                low = mid + 1;
            } else {
                high = mid - 1;
            }
        }
        return -1;
    }

    @Override // java.util.List
    public ListIterator listIterator() {
        return this.items.listIterator();
    }

    @Override // java.util.List
    public ListIterator listIterator(int index) {
        return this.items.listIterator(index);
    }

    @Override // java.util.List
    public Object remove(int index) {
        return this.items.remove(index);
    }

    @Override // java.util.List, java.util.Collection
    public boolean remove(Object o) {
        return this.items.remove(o);
    }

    @Override // java.util.List, java.util.Collection
    public boolean removeAll(Collection c) {
        boolean ret = false;
        Iterator itr = c.iterator();
        while (itr.hasNext()) {
            if (remove(itr.next())) {
                ret = true;
            }
        }
        return ret;
    }

    @Override // java.util.List, java.util.Collection
    public boolean retainAll(Collection c) {
        List list = new SortedList(c.size(), this.comparator);
        list.addAll(c);
        boolean ret = false;
        for (int i = size() - 1; i >= 0; i--) {
            if (!list.contains(get(i))) {
                remove(i);
                ret = true;
            }
        }
        return ret;
    }

    @Override // java.util.List
    public Object set(int index, Object element) {
        if (compareObjects(get(index), element) == 0) {
            return this.items.set(index, element);
        }
        Object ret = this.items.get(index);
        this.items.remove(index);
        add(element);
        return ret;
    }

    @Override // java.util.List, java.util.Collection
    public int size() {
        return this.items.size();
    }

    @Override // java.util.List
    public List subList(int fromIndex, int toIndex) {
        SortedList result = new SortedList(this.comparator);
        result.items = this.items.subList(fromIndex, toIndex);
        return result;
    }

    @Override // java.util.List, java.util.Collection
    public Object[] toArray() {
        return this.items.toArray();
    }

    @Override // java.util.List, java.util.Collection
    public Object[] toArray(Object[] a) {
        return this.items.toArray(a);
    }

    public Comparator getComparator() {
        return this.comparator;
    }

    public void setComparator(Comparator comparator) {
        this.comparator = comparator;
    }

    public boolean find(Object element, int[] index) {
        int low = 0;
        int high = size() - 1;
        while (low <= high) {
            int mid = (low + high) / 2;
            int flag = compareObjects(element, get(mid));
            if (flag == 0) {
                index[0] = mid;
                return true;
            }
            if (flag > 0) {
                low = mid + 1;
            } else {
                high = mid - 1;
            }
        }
        index[0] = low;
        return false;
    }

    public int find(Object element) {
        if (find(element, this.findStub)) {
            return this.findStub[0];
        }
        return -1;
    }

    public void reset(List items) {
        if (items == null) {
            throw new NullPointerException("items");
        }
        Collections.sort(items, new Comparator() { // from class: com.jiuqi.util.SortedList.1
            @Override // java.util.Comparator
            public int compare(Object o1, Object o2) {
                return SortedList.this.compareObjects(o1, o2);
            }
        });
        this.items = items;
    }
}
